/* Modifications Copyright 2000 Compaq Computer Corporation */
/*
 * @(#)typedefs_md.h	1.55 00/08/10
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

/*
 * Solaris-dependent types for Green threads
 */

#ifndef _JAVASOFT_SOLARIS_TYPES_MD_H_
#define _JAVASOFT_SOLARIS_TYPES_MD_H_

#include <sys/types.h>
#include <sys/stat.h>
/*** DEC Java: OSF/cds - added by OSF for Digital Unix: ***/
#include <math.h>
#include <float.h>
# if defined(__VMS)
#include <limits.h>	/*** DEC Java: JZ: added for VMS */
#endif

#ifdef __linux__
#include <stdint.h>
#define HAVE_INTPTR_T
#define _UINT64_T
#endif

/* #### DEC Java: BP this is defined below
#define int8_t char
*/

/* Fix for varargs differences on PowerPC */
#if defined(__powerpc__)
#define VARGS(x) (x)
#else
#define VARGS(x) (&x)
#endif /* __powerpc__ */

 
/* temporary scaffolding, to allow for back and forth testing */
/*** DEC Java: HDS: make sure int32_t and uint32_t are not defined as long's
#ifndef USE_REAL_INT32
#define int32_t long
#define uint32_t unsigned long
#endif
***/


/*** DEC Java: HDS change __alpha__ to __alpha ***/
#if defined(__alpha)
# if defined(__VMS)
#define PTR_IS_32 1
# else
#define PTR_IS_64 1
#define LONG_IS_64 1
# endif /* __VMS */
#else
#define PTR_IS_32 1
#endif

/* don't redefine typedef's on Solaris 2.6 or Later */
/*** DEC Java: JZ These types are defined in inttypes.h */
#ifdef __VMS
#ifndef __INTTYPES_LOADED
#include <inttypes.h>
#endif
#endif

/*** DEC Java: JZ, added __INTTYPES_LOADED ***/
#if !defined(_ILP32) && !defined(_LP64) && !defined(__INTTYPES_LOADED)

#ifndef HAVE_INTPTR_T
#ifdef LONG_IS_64
typedef long intptr_t;
typedef unsigned long uintptr_t;
#else
typedef int intptr_t;
typedef unsigned int uintptr_t;
#endif	/* LONG_IS_64 */
#endif /* don't HAVE_INTPTR_T */

#ifndef	_UINT64_T
#define	_UINT64_T
#ifdef LONG_IS_64
typedef unsigned long uint64_t;
#else
typedef unsigned long long uint64_t;
#endif
#define _UINT32_T
#ifndef uint32_t /* [sbb] scaffolding */
typedef unsigned int uint32_t;
#endif /* [sbb] scaffolding */
#if defined(__linux__)
typedef unsigned int uint_t;
#endif
#endif

#ifndef __BIT_TYPES_DEFINED__
/* that should get Linux, at least */
#ifndef	_INT64_T
#define	_INT64_T
#ifdef LONG_IS_64
typedef long int64_t;
#else
typedef long long int64_t;
#endif
#define _INT32_T
#ifndef int32_t /* [sbb] scaffolding */
typedef int int32_t;
#endif /* [sbb] scaffolding */
#if defined(__linux__)
typedef int int_t;
#endif
#endif
#endif /* __BIT_TYPES_DEFINED__ */

#endif	 /* !defined(_ILP32) && !defined(_LP64) */

/*** DEC Java: OSF/cds - short integer definitions added by OSF (16-bit and 8-bit) ***/
/*** DEC Java: JZ These types are defined in inttypes.h */

#if !defined(__INTTYPES_LOADED)
#ifndef _INT16_T
#ifndef __linux__ /* DEC Java HDS already defined for linux */
typedef short int16_t;
#endif
#define _INT16_T
#endif /* _INT16_T */

#ifndef _UINT16_T
#ifndef __linux__ /* DEC Java HDS already defined for linux */
typedef unsigned short uint16_t;
#endif
#define _UINT16_T
#endif /* _UINT16_T */

#ifndef _INT8_T
#ifndef __linux__ /* DEC Java HDS already defined for linux */
typedef signed char int8_t;     /* Plain char may be unsigned. */
#endif
#define _INT8_T
#endif /* _INT8_T */

#ifndef _UINT8_T
#ifndef __linux__ /* DEC Java HDS already defined for linux */
typedef unsigned char uint8_t;
#endif
#define _UINT8_T
#endif /* _UINT8_T */
#endif /* __INTTYPES_LOADED */

/* *** DEC Java: OSF/cds - added by OSF
 *  An unsigned integer of the same size as void* to perform mask
 *  and unmaks on addresses.  An invalid pointer recognizable at
 *  a glance.
 */
#ifndef _UINT_PTR_T
typedef unsigned long uint_ptr_t;
#define _UINT_PTR_T
#endif /* _UINT_PTR_T */

#define INVALID_POINTER 0x5555555555555555

/* use these macros when the compiler supports the long long type */

#define ll_high(a)    ((uint32_t)(((uint64_t)(a))>>32))
#define ll_low(a)     ((uint32_t)(a))
#define int2ll(a)	((int64_t)(a))
#define ll2int(a)	((int)(a))
#define ll_add(a, b)	((int64_t)(a) + (int64_t)(b))
#define ll_and(a, b)	((int64_t)(a) & (int64_t)(b))
#define ll_div(a, b)	((int64_t)(a) / (int64_t)(b))
#define ll_mul(a, b)	((int64_t)(a) * (int64_t)(b))
#define ll_neg(a)	(-(a))
#define ll_not(a)     (~(uint64_t)(a))
#define ll_or(a, b)   ((uint64_t)(a) | (b))
#define ll_shl(a, n)  ((uint64_t)(a) << (n))
#define ll_shr(a, n)  ((int64_t)(a) >> (n))
#define ll_sub(a, b)  ((uint64_t)(a) - (b))
#define ll_ushr(a, n) ((uint64_t)(a) >>(n))
#define ll_xor(a, b)  ((int64_t)(a) ^ (int64_t)(b))
#define uint2ll(a)    ((uint64_t)(a))
#define ll_rem(a,b)	((int64_t)(a) % (int64_t)(b))

extern int32_t float2l(float f);
extern int32_t double2l(double d);
extern int64_t float2ll(float f);
extern int64_t double2ll(double d);

#define ll2float(a)	((float) (a))
#define ll2double(a)	((double) (a))

/* Useful on machines where jlong and jdouble have different endianness. */
#define ll2double_bits(a)  ((void) 0)

/* comparison operators */
#define ll_ltz(ll)	((ll)<0)
#define ll_gez(ll)	((ll)>=0)
#define ll_eqz(a)	((a) == 0)
#define ll_nez(a)	((a) != 0)
#define ll_eq(a, b)	((a) == (b))
#define ll_ne(a,b)	((a) != (b))
#define ll_ge(a,b)	((a) >= (b))
#define ll_le(a,b)	((a) <= (b))
#define ll_lt(a,b)	((a) < (b))
#define ll_gt(a,b)	((a) > (b))

#define ll_zero_const	((int64_t) 0)
#define ll_one_const	((int64_t) 1)

extern void ll2str(int64_t a, char *s, char *limit);

#define ll2ptr(a) ((void*)(uintptr_t)(a))
#define ptr2ll(a) ((int64_t)(uintptr_t)(a))

/*** DEC Java: JZ Added __alpha */
#if defined(__alpha) || (defined(__linux__) && defined(ppc))
#ifndef HAVE_ALIGNED_DOUBLES
#define HAVE_ALIGNED_DOUBLES
#endif
#ifndef HAVE_ALIGNED_LONGLONGS
#define HAVE_ALIGNED_LONGLONGS
#endif
#endif

#endif /* !_JAVASOFT_SOLARIS_TYPES_MD_H_ */
